﻿-- General.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format(L["Module '%s' requires AshranBuddy"], L["General"]))
end

-- Registration
local General = AshranBuddy:RegisterModule("General")
LibStub("LibWho-2.0"):Embed(General)

-- Locals
local AshranMapID = 978
local AshranMapName = GetMapNameByID(AshranMapID)

local QueueStart = 0
local QueueTime = -1
local LastQueryTime = 0
local QueryIntervalTime = 10
local WaitTimes = { }

local MaxGroupMembers = 40
local BrokenBonesItemID = 118043

local GroupTypes = {
	Main = {
		Title = "Road of Glory",
		Keywords = { "inv", "invite" },
		Blacklist = { "event", "evnt" },
	},
	Events = {
		Title = "Zone Events",
		Keywords = { "event" },
		Blacklist = { },
	},
}
local GroupType = GroupTypes.Main

--TODO: EncounterInstanceButtonTemplate.bgImage
--TODO: QueueStatusFrame.StatusEntries

function General:OnInitialize()
	self:InitializeComponent()
	
	if (AshranBuddy.db.global.version ~= AshranBuddy.version) then
		self:ShowNewFeaturesComponent()
		AshranBuddy.db.global.version = AshranBuddy.version
	end
end

function General:OnEnable()
	self:RegisterEvent("CHAT_MSG_CHANNEL", "OnChatMessage")
	self:RegisterEvent("CHAT_MSG_SAY", "OnChatMessage")
	self:RegisterEvent("CHAT_MSG_WHISPER", "OnChatMessage")
	self:RegisterEvent("CHAT_MSG_YELL", "OnChatMessage")
    self:RegisterEvent("BATTLEFIELD_MGR_QUEUE_INVITE", "OnQueueInvite")
    self:RegisterEvent("BATTLEFIELD_MGR_ENTERED", "OnEntered")
	
    self:RegisterEvent("BATTLEFIELD_MGR_EJECT_PENDING", "OnQueueDebug")
    self:RegisterEvent("BATTLEFIELD_MGR_EJECTED", "OnQueueDebug")
    self:RegisterEvent("BATTLEFIELD_MGR_ENTRY_INVITE", "OnQueueDebug")
    self:RegisterEvent("BATTLEFIELD_MGR_QUEUE_REQUEST_RESPONSE", "OnQueueDebug")
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", General.OnChatMessageLootFilter)
end

function General:OnDisable()
	self:UnregisterAllEvents()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_LOOT", General.OnChatMessageLootFilter)
end

function General:ShowNewFeaturesComponent()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetToplevel(true)
	frame:SetFrameStrata("DIALOG")
	frame:EnableMouse(true)
	frame:SetSize(778, 810)
	frame:SetPoint("CENTER")
	frame:SetScale(0.9)
	frame:SetBackdrop({
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		tileSize = 32,
		edgeSize = 32,
		insets = { left = 11, right = 12, top = 12, bottom = 11 }
	})
	
	local background = frame:CreateTexture(nil, "BACKGROUND")
	background:SetTexture(0.105, 0.105, 0.105, 1.0)
	background:SetWidth(768)
	background:SetHeight(800)
	background:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
	background:SetDrawLayer("BACKGROUND", -1)
	
	local abnf = {
		{ relative = 0, to = "TOPLEFT", x = 5, y = -5 },
		{ relative = 1, to = "TOPRIGHT", x = 0, y = 0 },
		{ relative = 2, to = "TOPRIGHT", x = 0, y = 0 },
		{ relative = 1, to = "BOTTOMLEFT", x = 0, y = 0 },
		{ relative = 2, to = "BOTTOMLEFT", x = 0, y = 0 },
		{ relative = 3, to = "BOTTOMLEFT", x = 0, y = 0 },
		{ relative = 4, to = "BOTTOMLEFT", x = 0, y = 0 },
		{ relative = 5, to = "BOTTOMLEFT", x = 0, y = 0 },
		{ relative = 6, to = "BOTTOMLEFT", x = 0, y = 0 },
	}
	
	for i = 1, #abnf do
		local relative = nil
		if (abnf[i].relative == 0) then
			relative = frame
		else
			relative = abnf[abnf[i].relative].Texture
		end
		
		abnf[i].Texture = frame:CreateTexture(nil, "BACKGROUND")
		abnf[i].Texture:SetTexture("Interface\\AddOns\\AshranBuddy\\Content\\abnf_0"..tostring(i))
		abnf[i].Texture:SetWidth(256)
		abnf[i].Texture:SetHeight(256)
		abnf[i].Texture:SetPoint("TOPLEFT", relative, abnf[i].to, abnf[i].x, abnf[i].y)
	end
	
	local button = CreateFrame("Button", nil, frame, "GameMenuButtonTemplate")
	button:SetPoint("BOTTOMRIGHT", frame, -13, 14)
	button:SetText("Close")
	button:SetScript("OnClick", function(self)
		self:GetParent():Hide()
	end)
	
	frame:Show()
end

function General:OnChatMessage(event, message, sender, language, channelString, target, flags, arg1, channelNumber, channelName, arg2, counter)
	message = message:lower()
	if (IsInRaid() and GetNumGroupMembers() < MaxGroupMembers) then
		--if (message:any(GroupType.Keywords) and not message:any(GroupType.Blacklist)) then
		--end
	end
end

function General:OnChatMessageLootFilter(event, message)
	if (message ~= nil and message:find(BrokenBonesItemID)) then
		return true
	end
	return false
end

function General:OnQueueInvite(event)
	trace("--> OnQueueInvite")
	QueueStart = GetTime()
	QueueTime = -1
	--self.component:SetScript("OnUpdate", self.OnUpdate)
end

function General:OnEntered(event)
	trace("OnEntered -->")
	if (QueueStart == 0) then
		QueueStart = GetTime()
	end
	QueueTime = GetTime() - QueueStart
	--self.component:SetScript("OnUpdate", nil)
end

function General:OnQueueDebug(event)
	trace(event)
end

function General:OnUpdate(elapsed)
	if (GetTime() > LastQueryTime + QueryIntervalTime) then
		self:Who("-z "..AshranMapName, { queue = self.WHOLIB_QUERY_QUIET, callback = General.OnUpdateCallback })
		LastQueryTime = GetTime()
		QuerySent = true
	end
end

function General:OnUpdateCallback(query, results, complete)
	if (QuerySent) then
		QuerySent = false
		local count = 0
		for _, result in pairs(results) do
			count = count + 1
			self:RequestQueueTime(result.Name)
		end
		trace("Count: "..count)
	end
end

function General:UpdateQueueEntry()
	for _, entry in pairs(QueueStatusFrame.StatusEntries) do
		if (entry.Title:find(AshranMapName)) then
			local height = entry:GetHeight()
			local waitTime = self:CalculateWaitTime()
			
			if (not waitTime or waitTime <= 0) then
				entry.AverageWait:Hide()
			elseif (not entry.AverageWait:IsShown()) then
				entry.AverageWait:SetPoint("TOPLEFT", entry, "TOPLEFT", 10, - (height + 5))
				entry.AverageWait:Show()
				height = height + entry.AverageWait:GetHeight()
			end
			
			entry.AverageWait:SetFormattedText(LFG_STATISTIC_AVERAGE_WAIT, SecondsToTime(waitTime, false, false, 1))
			entry:SetHeight(height)
		end
	end
end

function General:CalculateWaitTime()
	local waitTime = 0
	local count = 0
	
	for sender, queueTime in pairs(WaitTimes) do
		waitTime = waitTime + queueTime
		count = count + 1
	end
	if (count <= 0) then
		return 0
	end
	
	return waitTime / count
end

function General:InitializeComponent()
end

function General:OnChatCommand(input)
end

function General:OnCreateOptions(options)
end

function General:OnMessageReceived(packet, distribution, sender)
	-- Someone has requested your queue time
	if (packet.type == "QT") then
		General:SendQueueTime(sender)
	end
	
	-- You have received a queue time from someone
	if (packet.type == "TS" and distribution == "WHISPER") then
		General:ReceiveQueueTime(sender, packet.data)
	end
end

function General:RequestQueueTime(sender)
	local packet = { type = "QT", data = { } }
	self:SendMessage(packet, "WHISPER", sender)
end

function General:SendQueueTime(sender)
	if (QueueTime ~= -1) then
		local packet = { type = "TS", data = { qt = QueueTime } }
		self:SendMessage(packet, "WHISPER", sender)
	end
end

function General:ReceiveQueueTime(sender, data)
	if (data ~= nil) then
		WaitTimes[sender] = data.qt
		self:UpdateQueueEntry()
	end
end